package com.asset.controller;

import com.asset.common.*;
import com.asset.domain.User;
import com.asset.mapper.UserMapper;
import com.asset.service.UserService;
import com.github.pagehelper.PageHelper;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.UsernamePasswordToken;
import org.apache.shiro.crypto.hash.Md5Hash;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/*
 * @author xiaol
 * @date 2020/1/3
 */
@RestController
public class UserController {

    @Autowired
    UserService userService;

    @Autowired
    UserMapper userMapper;

    @RequestMapping("/login")
    public Result login(@Validated(value={Login.class}) @RequestBody User user, BindingResult result){
        if (result.hasErrors()) {
            return ResultUtil.Validated(result);
        }
        try {
            String name = user.getUsername();
            String password = user.getPassword();
            password = new Md5Hash(password,name,3).toString();
            UsernamePasswordToken upToken = new UsernamePasswordToken(name,password);
            //1.获取subject
            Subject subject = SecurityUtils.getSubject();
            //获取session
            String sid = (String) subject.getSession().getId();
            //2.调用subject进行登录
            subject.login(upToken);
            return ResultUtil.success(CodeEnum.LOGIN_SUCCESS);
        }catch (Exception e) {
            return ResultUtil.error(CodeEnum.LOGIN_ERROR);
        }
    }

    @RequestMapping("/loginOut")
    public Result loginOut(){
        try {
            //1.获取subject
            Subject subject = SecurityUtils.getSubject();
            //2.调用subject进行登出
            subject.logout();
            return ResultUtil.success(CodeEnum.LOGINOUT_SUCCESS);
        }catch (Exception e) {
            return ResultUtil.error(CodeEnum.ERROR);
        }
    }

    @RequestMapping(value = "/user/getUserInfo")
    public Result getUserInfo() {
        User user = (User) SecurityUtils.getSubject().getPrincipal();
        return ResultUtil.success(CodeEnum.SUCCESS,user);
    }
    @RequestMapping(value = "/user/getUserById")
    public Result getUserById(int id) {
        Map<String,Object> user = userService.getUserByIdInfo(id);
        /*String hashAlgorithmName = "MD5";
        Object credentials = "123456";
        Object salt = "xiaol";
        int hashIterations = 3;
        Object result = new SimpleHash(hashAlgorithmName, credentials, salt, hashIterations);*/
        System.out.println(user);
        return ResultUtil.success(CodeEnum.SUCCESS,user);
    }

    @RequestMapping(value = "/user/getUserList")
    public Result getUserList(@RequestParam(defaultValue = "1") int page, @RequestParam(defaultValue = "10") int per,@RequestParam(value="username",required=false)String username) {
        PageHelper.startPage(page, per);
        Map param = new HashMap();
        param.put("username",username);
        List<User> lists = userService.getUserList(param);
        Map map = Pagination.Pagination(lists);
        return ResultUtil.success(CodeEnum.SUCCESS,map);
    }
    @RequestMapping(value = "/user/addUser")
    public Result addUser(@Validated(value={Insert.class}) @RequestBody User user, BindingResult result) {
        if (result.hasErrors()) {
            return ResultUtil.Validated(result);
        }
        if(userService.getUserByName(user.getUsername()) != null){
            return ResultUtil.error(CodeEnum.ISEXITE_ERROR);
        }
        System.out.println(user.getPassword());
        user.setCreate_time(Common.getTime());
        String password = Common.passwordHash(user.getPassword(),user.getUsername());
        user.setPassword(password);
        if (userService.insertUser(user)) {
            return ResultUtil.success(CodeEnum.SUCCESS);
        } else {
            return ResultUtil.error(CodeEnum.ERROR);
        }
    }

    @RequestMapping(value = "/user/updateUser")
    public Result updateUser(@Validated(value={Update.class}) @RequestBody User user, BindingResult result) {
        if (result.hasErrors()) {
            return ResultUtil.Validated(result);
        }
        System.out.println(user);
        if (userService.updateUser(user)) {
            return ResultUtil.success(CodeEnum.SUCCESS);
        } else {
            return ResultUtil.error(CodeEnum.ERROR);
        }
    }

    @RequestMapping(value = "/user/updatePassword")
    public Result updatePassword(@Validated(value={Special.class}) @RequestBody User user, BindingResult result) {
        if (result.hasErrors()) {
            return ResultUtil.Validated(result);
        }
        System.out.println(user);
        if (userService.updatePassword(user)) {
            return ResultUtil.success(CodeEnum.SUCCESS);
        } else {
            return ResultUtil.error(CodeEnum.ERROR);
        }
    }

    @RequestMapping(value = "/user/deleteUser")
    public Result deleteUserById(Integer id) {
        if (userService.deleteUserById(id)) {
            return ResultUtil.success(CodeEnum.SUCCESS);
        } else {
            return ResultUtil.error(CodeEnum.ERROR);
        }
    }

    @RequestMapping(value = "/autherror")
    public Result autherror(int code){
        return code == 0 ? ResultUtil.error(CodeEnum.UN_LOGIN) : ResultUtil.error(CodeEnum.AUTH_ERROR);
    }

    @RequestMapping(value = "/errorPage")
    public Result errorPage(int code,HttpServletRequest request){
        return code == 0 ? ResultUtil.success(CodeEnum.SUCCESS) : ResultUtil.error(CodeEnum.ERROR,getStatus(request));
    }

    private HttpStatus getStatus(HttpServletRequest request) {
        Integer statusCode = (Integer) request
                .getAttribute("javax.servlet.error.status_code");
        if (statusCode == null) {
            return HttpStatus.INTERNAL_SERVER_ERROR;
        }
        try {
            return HttpStatus.valueOf(statusCode);
        }
        catch (Exception ex) {
            return HttpStatus.INTERNAL_SERVER_ERROR;
        }
    }

}
