package com.asset.controller.system;

import com.asset.common.*;
import com.asset.domain.system.Company;
import com.asset.service.UserService;
import com.asset.service.system.CompanyService;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.google.zxing.client.j2se.MatrixToImageWriter;
import com.google.zxing.common.BitMatrix;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/*
 * @author xiaol
 * @date 2019/12/26
 */
@RestController
public class CompanyController {

    @Autowired
    CompanyService companyService;
    @Autowired
    UserService userService;

    //图片访问域名
    @Value("${tempContextUrl}")
    private String tempContextUrl;
    //图片存储目录
    @Value("${filePath}")
    private String filePath;

    @GetMapping("/getCompanyById")
    public Result getCompany(Integer id){
        Company company = companyService.getCompanyById(id);
        return ResultUtil.success(CodeEnum.SUCCESS,company);
    }

    @GetMapping(value = "listCompany")
    public Result companyList(@RequestParam(defaultValue = "1") int page, @RequestParam(defaultValue = "10") int per,String name) {
        PageHelper.startPage(page, per);
        Map param = new HashMap();
        param.put("name",name);
        Page<Company> list = companyService.companyList(param);
        Map map = Pagination.Pagination(list);
        return ResultUtil.success(CodeEnum.SUCCESS,map);
    }

    @PostMapping("/insertCompany")
    public Result insertCompany(@Validated(value={Insert.class}) @RequestBody Company company, BindingResult result){
        if (result.hasErrors()) {
            return ResultUtil.Validated(result);
        }
        System.out.println(Common.getTime());
        company.setCreateTime((int) Common.getTime());
        long ss = Common.getTime();
        System.out.println(Common.getDateToString(ss));
        companyService.insertCompany(company);
        return ResultUtil.success(CodeEnum.SUCCESS);
    }

    @PostMapping("/updateCompany")
    public Result updateCompany(@Validated(value={Update.class}) @RequestBody Company company, BindingResult result){
        if (result.hasErrors()) {
            return ResultUtil.Validated(result);
        }
        companyService.updateCompany(company);
        return ResultUtil.success(CodeEnum.SUCCESS);
    }

    @GetMapping("/deleteCompanyById")
    public Result deleteCompanyById(Integer id){
        if (companyService.deleteCompanyById(id)) {
            return ResultUtil.success(CodeEnum.SUCCESS);
        } else {
            return ResultUtil.error(CodeEnum.ERROR);
        }
    }

    @RequestMapping("/deleteCompanyByIds")
    public Result deleteCompanyByIds(String ids,String name){
        String[] arr = ids.split(",");
        List list = new ArrayList();
        for (int i=0;i<arr.length;i++){
            list.add(arr[i]);
        }
        System.out.println(list);
        if (companyService.deleteCompanyByIds(list,name)) {
            return ResultUtil.success(CodeEnum.SUCCESS);
        } else {
            return ResultUtil.error(CodeEnum.ERROR);
        }
    }

    /*
     * 上传文件
     */
    @RequestMapping("/upload")
    public Result upload(MultipartFile file) throws IOException {
        if(file!=null && file.getOriginalFilename()!=null && file.getOriginalFilename().length()>0){
            Upload upload = new Upload();
            Map map = upload.Upload(file, filePath, tempContextUrl,true);
            return ResultUtil.success(CodeEnum.SUCCESS,map);
        }
        return ResultUtil.error(CodeEnum.ERROR);
    }
    /*
     * 导入Excel
     */
    @RequestMapping("/importDo")
    public Result importDo(MultipartFile file) throws IOException {
        Excel excel = new Excel();
        List<List<List<String>>> listData = excel.fileType(file);
        List<Company> list = new ArrayList<Company>();
        for (int i = 0; i < listData.size(); i++) {
            for (int j =0; j < listData.get(i).size(); j++){
                Company company = new Company();
                company.setName(listData.get(i).get(j).get(0));
                company.setAddress(listData.get(i).get(j).get(1));
                company.setPhone(listData.get(i).get(j).get(2));
                company.setState(listData.get(i).get(j).get(3));
                company.setCreateTime((int) Common.getTime());
                list.add(company);
            }
        }
        if (companyService.insertAllCompany(list)) {
            return ResultUtil.success(CodeEnum.SUCCESS);
        } else {
            return ResultUtil.error(CodeEnum.ERROR);
        }
    }

    /*
     *导出
     */
    @RequestMapping("/downloadDo")
    public void downloadDo(HttpServletResponse response, String name) throws Exception {
        XSSFWorkbook tmpWorkbook = new XSSFWorkbook();
        String[] tmpUserTitles = {"序号", "公司名称", "地址", "电话", "状态", "创建时间"};
        List<Company> companyLists = (List<Company>) companyService.getCompanyByName(name);
        List<String[]> companys = new ArrayList<>();
        for (Company companyList : companyLists) {
            String[] company = new String[5];
            company[0] = String.valueOf(companyList.getId());
            company[1] = companyList.getName();
            company[2] = companyList.getAddress();
            company[3] = companyList.getPhone();
            company[4] = companyList.getState();
            company[5] = String.valueOf(companyList.getCreateTime());
            companys.add(company);
        }
        Excel excel = new Excel();
        excel.createSheetData(tmpWorkbook, tmpUserTitles, "列表", companys);
        String filename = "列表" + Common.getDate() + ".xlsx";
        excel.setResponseHeader(response, filename);
        OutputStream tmpOutputStream = response.getOutputStream();
        tmpWorkbook.write(tmpOutputStream);
        tmpOutputStream.flush();
        tmpOutputStream.close();
    }

    /*
     *生成二维码(直接显示页面上)
     */
    @RequestMapping("/generateQRCode")
    public void generateQRCode(HttpServletResponse response,String name) {
        String longUrl;
        try {
            longUrl = "http://www.ecrrc.com" + name;
            // 生成二维码
            BitMatrix qRcodeImg = QRCodeUtil.generateQRCodeStream(longUrl, response,300,300);
            // 将二维码输出到页面中
            MatrixToImageWriter.writeToStream(qRcodeImg, "png", response.getOutputStream());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /*
     *生成二维码(指定路径生成图片)
     */
    @RequestMapping("/createQRCode")
    public Result createQRCode(String name) throws Exception {
        String text = "http://www.baidu.com";
        String destPath = "D:\\phpStudy\\WWW\\images\\";
        QRCode.encode(text, null, destPath, true);
        return ResultUtil.success(CodeEnum.SUCCESS);
    }

}
